@@ -11,6 +11,7 @@
 #include <QLabel>
 #include <QDebug>
 #include <QGraphicsSceneMouseEvent>
+#include <QtConcurrentRun>
 
 
 
@@ -508,6 +509,7 @@ QDeclarativeItem(parent) {
 	curRegionIdx = -1;
 	context = NULL;
 	document = NULL;
+	page = NULL;
 	pixmap = NULL;
 	text_page = NULL;
 	pageobj = NULL;
@@ -523,6 +525,9 @@ QDeclarativeItem(parent) {
 	clickTimer.setInterval(400);
 	clickTimer.setSingleShot(true);
 	connect(&clickTimer, SIGNAL(timeout()), this, SIGNAL(clickForward()));
+	renderTimer.setInterval(10);
+	renderTimer.setSingleShot(true);
+	connect(&renderTimer, SIGNAL(timeout()), this, SLOT(render()));
 	
 	textDocument = QList<QString>();
 	hitCount = QList<int>();
@@ -531,6 +536,7 @@ QDeclarativeItem(parent) {
 	ct << QColor(137,153,137,255).rgba ();
 	ct << 0;
 	noteImage = QImage(":/note_inv.png").scaled(24,24).convertToFormat(QImage::Format_Indexed8, ct);
+	loadImage = QImage(":/load.png");
 	
 	image = new QImage(800, 800, QImage::Format_RGB32);
 	QRgb value;
@@ -684,13 +690,18 @@ void QPdfImage::writeCurrentFile(){
 	}
 }
 
-bool QPdfImage::gotoPage(int pagenum,bool render){
+
+#include <QCoreApplication>
+bool QPdfImage::gotoPage(int pagenum,bool doRender){
 	if(!document || pagenum > totalpages_prv || pagenum < 1)
 		return false;
 	
 	writeAnnotationsToPage();
 	
-	fz_page *page = fz_load_page(document, pagenum - 1);
+	//delete the page reference
+	if(page)
+		fz_free_page(document, page);
+	page = fz_load_page(document, pagenum - 1);
 	pageobj = ((pdf_document*) document)->page_objs[pagenum-1];
 	
 	//render page to pixmap
@@ -700,7 +711,7 @@ bool QPdfImage::gotoPage(int pagenum,bool render){
 	originalWidth_prv = rect.x1 - rect.x0;
 	originalHeight_prv = rect.y1 - rect.y0;
 	rect = fz_transform_rect(transform, rect);
-	fz_bbox bbox = fz_round_rect(rect); /*bounding box is int,
+	/*fz_bbox bbox = fz_round_rect(rect); /*bounding box is int,
 	fz_round_rect may be smaller than fz_bbox_covering_rect */
 	
 	
@@ -733,35 +744,15 @@ bool QPdfImage::gotoPage(int pagenum,bool render){
 		fz_links = fz_links->next;
 	}
 	
-	
-	if(render){
-		//save the pointer to the old pixmap
-		fz_pixmap* oldpm = pixmap;
-		
-		pixmap = fz_new_pixmap_with_bbox(context, fz_device_bgr, bbox);//bgr matches Qt's rgb!
-		fz_clear_pixmap_with_value(context, pixmap, 255);//draw on white background
-		fz_device *device = fz_new_draw_device(context, pixmap);
-		fz_run_page(document, page, device, transform, NULL);
-		fz_free_device(device);
-		
-		//prepare the raw image
-		unsigned char *samples = fz_pixmap_samples(context, pixmap);
-		int width = fz_pixmap_width(context, pixmap);
-		int height = fz_pixmap_height(context, pixmap);
-		
-		if(image)
-			delete image;
-		image = new QImage(samples, width, height, QImage::Format_ARGB32);
-		setSize(image->size());
-
-		readAnnotationsFromPage();
-		pageModifications = 0;
-		
-		update(boundingRect());
-		
-		//delete old pixmap
-		fz_drop_pixmap(context, oldpm);
+	if(doRender){
+		isLoading = true;
+		update();
+		QCoreApplication::processEvents();
+		renderTimer.start();
 	}
+// 	QFuture<void> fut;
+// 	if(doRender)  fut = QtConcurrent::run(this,&QPdfImage::render);
+// 	fut.waitForFinished();
 
 	if(pagenum_prv != pagenum){
 		pagenum_prv = pagenum;
@@ -770,12 +761,58 @@ bool QPdfImage::gotoPage(int pagenum,bool render){
 	
 	emit originalWidthChanged();
 	emit originalHeightChanged();
-	//delete the page reference
-	fz_free_page(document, page);
 	return true;
 }
 
+void QPdfImage::render(){
+	fz_matrix transform = fz_scale(zoom_prv/100.0,zoom_y_prv/100.0);
+	transform = fz_concat(transform, fz_rotate(0));
+	fz_bbox bbox = fz_round_rect(fz_transform_rect(transform,fz_bound_page(document, page)));
+
+	//save the pointer to the old pixmap
+	fz_pixmap* oldpm = pixmap;
+	
+	pixmap = fz_new_pixmap_with_bbox(context, fz_device_bgr, bbox);//bgr matches Qt's rgb!
+	fz_clear_pixmap_with_value(context, pixmap, 255);//draw on white background
+	fz_device *device = fz_new_draw_device(context, pixmap);
+	fz_run_page(document, page, device, transform, NULL);
+	fz_free_device(device);
+	
+	//prepare the raw image
+	unsigned char *samples = fz_pixmap_samples(context, pixmap);
+	int width = fz_pixmap_width(context, pixmap);
+	int height = fz_pixmap_height(context, pixmap);
+	
+	if(image)
+		delete image;
+	image = new QImage(samples, width, height, QImage::Format_ARGB32);
+	setSize(image->size());
+
+	readAnnotationsFromPage();
+	pageModifications = 0;
+	
+	isLoading = false;
+	emit renderingFinished();
+	update();
+	
+	//delete old pixmap
+	fz_drop_pixmap(context, oldpm);
+}
+
 void QPdfImage::paint(QPainter *painter, const QStyleOptionGraphicsItem*, QWidget*) {
+	if(isLoading){
+		painter->setPen(Qt::black);
+		painter->setBrush(Qt::black);
+		QFont fnt = painter->font();
+		fnt.setPixelSize(width()/20);
+		painter->setFont(fnt);
+		painter->drawImage(QPoint(0,0),loadImage.scaled(width(),height()));
+		painter->drawText(QRect(0.45*width(),0.4*height(),0.3*width(),0.3*height()),
+			Qt::AlignLeft|Qt::TextDontClip|Qt::TextSingleLine,
+			"loading ...");
+		return;
+	}
+		
 	if(textMode_prv){
 		painter->setPen(Qt::black);
 		painter->setBrush(Qt::black);
